# Copyright 2025 Enactic, Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.


"""
OpenArm CAN Python bindings for motor control via SocketCAN.

This package provides Python bindings for the OpenArm motor control system,
allowing you to control DAMIAO motors through SocketCAN.
"""

from .openarm_can import *

__version__ = "1.2.2"
__author__ = "Enactic, Inc."

# Direct export of C++ classes - no wrappers
__all__ = [
    # Enums
    "MotorType",
    "MotorVariable",
    "CallbackMode",

    # Data structures
    "LimitParam",
    "ParamResult",
    "MotorStateResult",
    "CanFrame",
    "CanFdFrame",
    "MITParam",

    # Main C++ classes (1:1 mapping)
    "Motor",
    "MotorControl",
    "CANSocket",           # Low-level socket with file descriptor access
    "CANDevice",           # Base CAN device class
    "MotorDeviceCan",      # Motor device management
    "CANDeviceCollection",  # Device collection management

    # Exceptions
    "CANSocketException",
]



# Copyright 2025 Enactic, Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import openarm_can as oa
import time
# Create OpenArm instance

arm = oa.OpenArm("can0", True)

# Initialize arm motors
motor_types = [oa.MotorType.DM4310, oa.MotorType.DM4310]
send_ids = [0x01, 0x02]
recv_ids = [0x11, 0x12]
arm.init_arm_motors(motor_types, send_ids, recv_ids)

# Initialize gripper
arm.init_gripper_motor(oa.MotorType.DM4310, 0x7, 0x17)
arm.set_callback_mode_all(oa.CallbackMode.IGNORE)
# Use high-level operations
arm.enable_all()
arm.recv_all()


# return to zero position
arm.set_callback_mode_all(oa.CallbackMode.STATE)
arm.get_arm().mit_control_all([oa.MITParam(2, 0.5, 0, 0, 0),
                               oa.MITParam(2, 0.5, 0, 0, 0)])

arm.recv_all()

# torque control test

arm.get_gripper().mit_control_all([oa.MITParam(0, 0, 0, 0, 0.15)])
arm.get_arm().mit_control_all(
    [oa.MITParam(0, 0, 0, 0, 0.15), oa.MITParam(0, 0, 0, 0, 0.15)])
arm.recv_all()

# read motor position
while True:
    arm.refresh_all()
    arm.recv_all()
    for motor in arm.get_arm().get_motors():
        print(motor.get_position())
    for motor in arm.get_gripper().get_motors():
        print(motor.get_position())



# Copyright 2025 Enactic, Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

# If you can use build, you should use pyproject.toml intead of
# setup.py. setup.py is only for old environment such as deb packages
# for Ubuntu 22.04.

import pathlib

from setuptools import setup, Extension
from setuptools.command.build_ext import build_ext


class cmake_build_ext(build_ext):
    def build_extensions(self):
        self.extensions = [
            ext for ext in self.extensions if ext.name != '__dummy__']
        super().run()

    def run(self):
        self._run_cmake()
        super().run()

    def _run_cmake(self):
        source = pathlib.Path().absolute()
        build_lib = pathlib.Path(self.build_lib).absolute()
        build_temp = pathlib.Path(self.build_temp).absolute()
        build_temp.mkdir(parents=True, exist_ok=True)
        self.spawn([
            "cmake",
            "-S", str(source),
            "-B", str(build_temp),
            "-DCMAKE_BUILD_TYPE=Release",
            f"-DCMAKE_INSTALL_PREFIX={build_lib}",
        ])
        self.spawn(["cmake", "--build", str(build_temp)])
        self.spawn(["cmake", "--install", str(build_temp)])


setup(
    name="openarm_can",
    version="1.2.2",
    license="Apache-2.0",
    license_files=["LICENSE.txt"],
    ext_modules=[Extension('__dummy__', [])],
    cmdclass={
        "build_ext": cmake_build_ext,
    }
)
